package java.io;

import sun.nio.ch.FileChannelImpl;
import java.nio.channels.FileChannel;

public class FileInputStream extends InputStream
{
    private final FileDescriptor fd;
    private final String path;
    private FileChannel channel;
    private final Object closeLock;
    private volatile boolean closed;
    
    public FileInputStream(String s) throws FileNotFoundException {
        this((s != null) ? new File(s) : null);
    }
    
    public FileInputStream(File file) throws FileNotFoundException {
        this.channel = null;
        this.closeLock = new Object();
        this.closed = false;
        String s = (file != null) ? file.getPath() : null;
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkRead(s);
        }
        if (s == null) {
            throw new NullPointerException();
        }
        if (file.isInvalid()) {
            throw new FileNotFoundException("Invalid file path");
        }
        (this.fd = new FileDescriptor()).attach(this);
        this.open(this.path = s);
        return;
    }
    
    public FileInputStream(FileDescriptor fileDescriptor) {
        this.channel = null;
        this.closeLock = new Object();
        this.closed = false;
        SecurityManager securityManager = System.getSecurityManager();
        if (fileDescriptor == null) {
            throw new NullPointerException();
        }
        if (securityManager != null) {
            securityManager.checkRead(fileDescriptor);
        }
        this.fd = fileDescriptor;
        this.path = null;
        this.fd.attach(this);
        return;
    }
    
    private native void open0(String p0) throws FileNotFoundException;
    
    private void open(String s) throws FileNotFoundException {
        this.open0(s);
    }
    
    @Override
    public int read() throws IOException {
        return this.read0();
    }
    
    private native int read0() throws IOException;
    
    private native int readBytes(byte[] p0, int p1, int p2) throws IOException;
    
    @Override
    public int read(byte[] array) throws IOException {
        return this.readBytes(array, 0, array.length);
    }
    
    @Override
    public int read(byte[] array, int n, int n2) throws IOException {
        return this.readBytes(array, n, n2);
    }
    
    @Override
    public long skip(long n) throws IOException {
        return this.skip0(n);
    }
    
    private native long skip0(long p0) throws IOException;
    
    @Override
    public int available() throws IOException {
        return this.available0();
    }
    
    private native int available0() throws IOException;
    
    @Override
    public void close() throws IOException {
        synchronized (this.closeLock) {
            if (this.closed) {
                return;
            }
            this.closed = true;
        }
        if (this.channel != null) {
            this.channel.close();
        }
        this.fd.closeAll(new Closeable() {
            @Override
            public void close() throws IOException {
                FileInputStream.access$000(FileInputStream.this);
            }
        });
    }
    
    public final FileDescriptor getFD() throws IOException {
        if (this.fd != null) {
            return this.fd;
        }
        throw new IOException();
    }
    
    public FileChannel getChannel() {
        synchronized (this) {
            if (this.channel == null) {
                this.channel = FileChannelImpl.open(this.fd, this.path, true, false, this);
            }
            return this.channel;
        }
    }
    
    private static native void initIDs();
    
    private native void close0() throws IOException;
    
    @Override
    protected void finalize() throws IOException {
        if (this.fd != null && this.fd != FileDescriptor.in) {
            this.close();
        }
    }
    
    static void access$000(FileInputStream fileInputStream) throws IOException {
        fileInputStream.close0();
    }
    
    static {
        initIDs();
    }
}
